/**
 * Construct a behavior factory that will conditionally apply a stylesheet based
 * on a MediaQueryList
 *
 * @param query - The MediaQueryList to subscribe to matches for.
 *
 * @public
 */
export function matchMediaStylesheetBehaviorFactory(query) {
    const cache = new WeakMap();
    return (sheet) => {
        return Object.freeze({
            query,
            cache,
            sheet,
            constructListener(source, sheet) {
                let attached = false;
                function listener() {
                    const { matches } = this;
                    if (matches && !attached) {
                        source.$fastController.addStyles(sheet);
                        attached = matches;
                    }
                    else if (!matches && attached) {
                        source.$fastController.removeStyles(sheet);
                        attached = matches;
                    }
                }
                return listener;
            },
            bind(source) {
                const { constructListener, query, cache } = this;
                const listener = constructListener(source, this.sheet);
                const cached = cache.get(source);
                // Invoke immediately to add if the query currently matches
                listener.bind(query)();
                query.addListener(listener);
                if (cached !== void 0) {
                    // Support multiple bindings of the same behavior
                    if (Array.isArray(cached)) {
                        cached.push(listener);
                    }
                    else {
                        cache.set(source, [cached, listener]);
                    }
                }
                else {
                    cache.set(source, listener);
                }
            },
            unbind(source) {
                const { cache, query } = this;
                const cached = cache.get(source);
                if (cached !== void 0) {
                    if (Array.isArray(cached)) {
                        cached.forEach(listener => query.removeListener(listener));
                    }
                    else {
                        query.removeListener(cached);
                    }
                    cache.delete(source);
                }
            },
        });
    };
}
/**
 * Implements {@link matchMediaStylesheetBehaviorFactory} for the {@link https://developer.mozilla.org/en-US/docs/Web/CSS/@media/forced-colors | forced-colors} query.
 * This can be used to construct a behavior to apply a forced-colors only stylesheet.
 * @public
 */
export const forcedColorsStylesheetBehavior = matchMediaStylesheetBehaviorFactory(window.matchMedia("(forced-colors)"));
